# coding: utf-8

from __future__ import absolute_import
from bs4 import BeautifulSoup
from typing import List, Dict
from stylelens_crawl_amazon.model import Item
from stylelens_crawl_amazon.model import ItemImage
from stylelens_crawl_amazon.model import ItemPrice


class ItemLookup(object):
  def __init__(self,
               amazon,
               item_ids,
               include_reviews_summary=False,
               response_groups=None):

    self._amazon = amazon
    self._item_ids = item_ids
    self._response_groups = response_groups
    self._include_reviews_summary = include_reviews_summary
    self._items = []

  def lookup(self):
    res = self._amazon.ItemLookup(ItemId=','.join(self._item_ids),
                            IncludeReviewsSummary=self._include_reviews_summary,
                            ResponseGroup=self._response_groups)
    soup = BeautifulSoup(res, "xml")
    # print(soup.prettify())
    # self._log(soup.prettify())

    items = soup.find('Items')
    request = items.find('Request')
    isValid = request.IsValid.text
    if isValid != 'True':
      print("isValid is not True")
      return None

    if items.Item:
      item = items.Item

    while True:
      self._extract_item(item)

      item = item.next_sibling
      if not item:
        break
    return self._items

  def _extract_item(self, data, parent_detail_page=None, parent_add_to_wishlist_link=None):
    item = Item()
    item.asin = data.ASIN.text
    if data.ParentASIN:
      item.parent_asin = data.ParentASIN.text

    if data.DetailPageURL:
      item.detail_page_link = data.DetailPageURL.text
    elif parent_detail_page != None:
      item.detail_page_link = parent_detail_page.replace(item.parent_asin, item.asin)

    if data.ItemLinks:
      item_link = data.ItemLinks.ItemLink
      while True:
        if item_link.Description:
          if 'Add To Wishlist' == item_link.Description.text:
            item.add_to_wishlist_link = item_link.URL.text
        item_link = item_link.next_sibling
        if not item_link:
          break
    elif parent_add_to_wishlist_link != None:
      item.add_to_wishlist_link = parent_add_to_wishlist_link.replace(item.parent_asin, item.asin)

    self._extract_images(item, data)

    if data.ItemAttributes:
      self._extract_item_attributes(item, data.ItemAttributes)

    if item.m_image is not None:
      self._items.append(item)

    if data.Variations:
      self._extract_variations(data.Variations, item.detail_page_link, item.add_to_wishlist_link)

    # if data.SimilarProducts:
    #   similar_products = data.SimilarProducts.find_all('SimilarProduct')
    #   for product in similar_products:
    #     if product.ASIN:
    #       self._similar_item_ids.append(product.ASIN.text)

  def _extract_variations(self, variations, parent_detail_page, parent_add_to_wishlist_link):

    if variations.Item:
      item = variations.Item
    else:
      return

    while True:
      self._extract_item(item, parent_detail_page=parent_detail_page, parent_add_to_wishlist_link=parent_add_to_wishlist_link)
      item = item.next_sibling
      if not item:
        break

  def _extract_images(self, item_attributes, data):
    if data.SmallImage:
      ii = ItemImage()
      ii.url = data.SmallImage.URL.text
      ii.height = data.SmallImage.Height.text
      ii.width = data.SmallImage.Width.text
      ii.unit = data.SmallImage.Width['Units']
      item_attributes.s_image = ii

    if data.MediumImage:
      ii = ItemImage()
      ii.url = data.MediumImage.URL.text
      ii.height = data.MediumImage.Height.text
      ii.width = data.MediumImage.Width.text
      ii.unit = data.MediumImage.Width['Units']
      item_attributes.m_image = ii

    if data.LargeImage:
      ii = ItemImage()
      ii.url = data.LargeImage.URL.text
      ii.height = data.LargeImage.Height.text
      ii.width = data.LargeImage.Width.text
      ii.unit = data.LargeImage.Width['Units']
      item_attributes.l_image = ii

  def _extract_item_attributes(self, item, data):
    if data.Binding:
      item.binding = data.Binding.text
    if data.Brand:
      item.brand = data.Brand.text
    if data.Department:
      item.department = data.Department.text
    if data.Color:
      item.color = data.Color.text
    if data.ClothingSize:
      item.clothing_size = data.ClothingSize.text
    if data.ListPrice:
      p = ItemPrice()
      p.amount = data.ListPrice.Amount.text
      p.currency_code = data.ListPrice.CurrencyCode.text
      p.formatted_price = data.ListPrice.FormattedPrice.text

    if data.Feature:
      item.features = []
      feature = data.Feature
      while True:
        item.features.append(feature.text)
        feature = feature.next_sibling
        if feature.name != 'Feature':
          break

    if data.ProductGroup:
      item.product_group = data.ProductGroup.text
    if data.ProductTypeName:
      item.product_type_name = data.ProductTypeName.text
    if data.Title:
      item.title = data.Title.text

  @property
  def item_ids(self):
    return self._item_ids
  # item_ids.__annotations__ = {'return': List[str]}

  @item_ids.setter
  def item_ids(self, item_ids):
    self._item_ids = item_ids
  # item_ids.__annotations__ = {'item_ids': List[str]}

  @property
  def search_index(self):
    return self._search_index
  # search_index.__annotations__ = {'return': str}

  @search_index.setter
  def search_index(self, search_index):
    self._search_index = search_index
  # search_index.__annotations__ = {'search_index': str}

  @property
  def response_groups(self):
    return self._response_groups
  # response_groups.__annotations__ = {'return': str}

  @response_groups.setter
  def response_groups(self, response_groups):
    self._response_groups = response_groups
  # response_groups.__annotations__ = {'response_groups': str}

  @property
  def browse_node(self):
    return self._browse_node
  # browse_node.__annotations__ = {'return': str}

  @browse_node.setter
  def browse_node(self, browse_node):
    self._browse_node = browse_node
  # browse_node.__annotations__ = {'browse_node': str}

  @property
  def sort(self):
    """
    Sort values
     - ref. : https://docs.aws.amazon.com/ko_kr/AWSECommerceService/latest/DG/LocaleUS.html
    relevancerank
    popularity-rank
    price
    -price
    reviewrank
    launch-date
    """
    return self._sort
  # sort.__annotations__ = {'return': str}

  @sort.setter
  def sort(self, sort):
    self._sort = sort
  # sort.__annotations__ = {'sort': str}

  @property
  def item_page(self):
    return self._item_page
  # item_page.__annotations__ = {'return': str}

  @item_page.setter
  def item_page(self, item_page):
    self._item_page = item_page
  # item_page.__annotations__ = {'item_page': str}

  def _log(self, data):
    f = open('lookup_log.txt', 'a', encoding='UTF-8')
    f.write(str(data))
    f.close()



# coding: utf-8

from __future__ import absolute_import
from .base_model_ import Model
from datetime import date, datetime
from typing import List, Dict
from ..util import deserialize_model
import traceback


class ItemSearchData(Model):
  def __init__(self,
               keywords=None,
               search_index=None,
               response_groups=None,
               browse_node=None,
               sort=None):
    self.bl_types = {
      'keywords': str,
      'search_index': str,
      'response_groups': str,
      'browse_node': str,
      'sort': str
    }

    self.attribute_map = {
      'keywords': 'keywords',
      'search_index': 'search_index',
      'response_groups': 'response_groups',
      'browse_node': 'browse_node',
      'sort': 'sort'
    }

    self._keywords = keywords
    self._search_index = search_index
    self._response_groups = response_groups
    self._browse_node = browse_node
    self._sort = sort

  @classmethod
  def from_dict(cls, dikt):
    return deserialize_model(dikt, cls)
  # from_dict.__annotations__ = {'return': 'ItemSearchData'}

  @property
  def keywords(self):
    return self._keywords
  # keywords.__annotations__ = {'return': str}

  @keywords.setter
  def keywords(self, keywords):
    self._keywords = keywords
  # keywords.__annotations__ = {'keywords': str}

  @property
  def search_index(self):
    return self._search_index
  # search_index.__annotations__ = {'return': str}

  @search_index.setter
  def search_index(self, search_index):
    self._search_index = search_index
  # search_index.__annotations__ = {'search_index': str}

  @property
  def response_groups(self):
    return self._response_groups
  # response_groups.__annotations__ = {'return': str}

  @response_groups.setter
  def response_groups(self, response_groups):
    self._response_groups = response_groups
  # response_groups.__annotations__ = {'response_groups': str}

  @property
  def browse_node(self):
    return self._browse_node
  # browse_node.__annotations__ = {'return': str}

  @browse_node.setter
  def browse_node(self, browse_node):
    self._browse_node = browse_node
  # browse_node.__annotations__ = {'browse_node': str}

  @property
  def sort(self):
    """
    Sort values
     - ref. : https://docs.aws.amazon.com/ko_kr/AWSECommerceService/latest/DG/LocaleUS.html
    relevancerank
    popularity-rank
    price
    -price
    reviewrank
    launch-date
    """
    return self._sort
  # sort.__annotations__ = {'return': str}

  @sort.setter
  def sort(self, sort):
    self._sort = sort
  # sort.__annotations__ = {'sort': str}




import bottlenose
import time
import random
from future.standard_library import install_aliases
install_aliases()

from urllib.parse import urlparse, urlencode
from urllib.request import urlopen, Request
from urllib.error import HTTPError
from .search_factory import SearchFactory


class AwsProductApi(object):
  def __init__(self, generate_item_searches=False):
    self._amazon = bottlenose.Amazon(ErrorHandler=self._error_handler)
    self._search_factory = SearchFactory(self._amazon, generate_item_searches)

  def _error_handler(self, err):
    ex = err['exception']
    if isinstance(ex, HTTPError) and ex.code == 503:
      time.sleep(random.expovariate(0.5))
      return True

  def get_item_searches(self):
    return self._search_factory.get_item_searches()

  def get_similar_items(self):
    return self._search_factory.get_similar_items()

  def item_search(self, item_searches):
    return self._search_factory.search(item_searches)

  def item_lookups(self):
    return self._search_factory.lookup_similar_items()



# coding: utf-8

from __future__ import absolute_import
from .base_model_ import Model
from datetime import date, datetime
from typing import List, Dict
from ..util import deserialize_model


class ItemPrice(Model):
  def __init__(self,
               amount=None,
               currency_code=None,
               formatted_price=None):
    self.bl_types = {
      'amount': str,
      'currency_code': str,
      'formatted_price': str
    }

    self.attribute_map = {
      'amount': 'amount',
      'currency_code': 'currency_code',
      'formatted_price': 'formatted_price'
    }

    self._amount = amount
    self._currency_code = currency_code
    self._formatted_price = formatted_price

  @classmethod
  def from_dict(cls, dikt):
    return deserialize_model(dikt, cls)
  # from_dict.__annotations__ = {'return': 'ItemPrice'}

  @property
  def amount(self):
    return self._amount
  # amount.__annotations__ = {'return': str}

  @amount.setter
  def amount(self, amount):
    self._amount = amount
  # amount.__annotations__ = {'amount': str}

  @property
  def currency_code(self):
    return self._currency_code
  # currency_code.__annotations__ = {'return': str}

  @currency_code.setter
  def currency_code(self, currency_code):
    self._currency_code = currency_code
  # currency_code.__annotations__ = {'currency_code': str}

  @property
  def formatted_price(self):
    return self._formatted_price
  # formatted_price.__annotations__ = {'return': str}

  @formatted_price.setter
  def formatted_price(self, formatted_price):
    self._formatted_price = formatted_price
  # formatted_price.__annotations__ = {'formatted_price': str}



# coding: utf-8

from __future__ import absolute_import

# import models into model package
from .item import Item
from .item_search_data import ItemSearchData
from .item_image import ItemImage
from .item_price import ItemPrice
